; ============================================================================
; 
; Author: Pascal Langer
; Web site: http://5xproject.dyndns.org/
;
; ============================================================================
; 
; Peripheral Description:
; 
; Firmware for the MM2001 hardware. Full compatible with IPL5X and LPT port.
; 
; ============================================================================
;
; Software Licence Agreement:
; 
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>;
;

	LIST P=16f874A
	#include "p16F874A.inc"
	errorlevel 1, -305

; Changed config bits
	__CONFIG    _CP_OFF & _DEBUG_OFF & _WRT_OFF & _CPD_OFF & _LVP_ON & _BODEN_ON  & _PWRTE_ON  & _WDT_OFF & _HS_OSC

FULL_STEP		EQU	0x20
INC_STEP		EQU	0x21
DEC_STEP		EQU	0x22
M1_STEP			EQU	0x23
M2_STEP			EQU	0x24
M3_STEP			EQU	0x25
M4_STEP			EQU	0x26
MOTORS			EQU	0x27
		
PORTCH			EQU	0x28
PORTCL			EQU	0x29
PORTDH			EQU	0x2A
PORTDL			EQU	0x2B
DIRS			EQU	0x2C
		
TABLE_STEP0		EQU	0x30
TABLE_STEP1		EQU	0x31
TABLE_STEP2		EQU	0x32
TABLE_STEP3		EQU	0x33
TABLE_STEP4		EQU	0x34
TABLE_STEP5		EQU	0x35
TABLE_STEP6		EQU	0x36
TABLE_STEP7		EQU	0x37
		
FREQ			EQU	0x40
PWM_PC			EQU	0x41
PWM_VAL			EQU	0x42
PWM_POS			EQU	0x43
PWM_MAX			EQU	0x44
PWM_VAL_CHANGED	EQU	0x45
PWM_MAX_CHANGED	EQU	0x46
S_TIMER			EQU 0x47
		
KEY_FILTER		EQU	0x48
KEY_FILTER_VAL	EQU	0x49
NBR_BIP			EQU	0x4A
WAITL			EQU	0x4B
WAITH			EQU	0x4C
WAITS			EQU	0x4D

STEPS_OLD		EQU	0x7C
STEPS_CHANGE	EQU	0x7D
STATUS_TEMP		EQU	0x7E
W_TEMP			EQU	0x7F

	ORG		0x00
	CLRF	STATUS				; Bank 0
	CLRF	PCLATH				; goto and call zone 0x000-0x7FF
	GOTO	Init
	NOP
;	ORG		0x04				; comment for MMUSB compatibility
	MOVWF	W_TEMP				; Copy W to TEMP register
	SWAPF	STATUS,W			; Swap status to be saved into W
	CLRF	STATUS				; bank 0, regardless of current bank, Clears IRP,RP1,RP0
	MOVWF	STATUS_TEMP			; Save status to bank zero STATUS_TEMP register

	BTFSS	INTCON, RBIF		;?PORTB INT
	GOTO	Isr_CCP1			; -> No
Isr_RB
	MOVF	PORTB,W
	BCF		INTCON, RBIF
	BTFSS	PORTB,0				; ?MOTEUR_ON
	GOTO	Isr_No_Steps		;   0=OFF
	ANDLW	0xF0				; Extract steps
	MOVWF	STEPS_CHANGE
	XORWF	STEPS_OLD,W
	XORWF	STEPS_OLD,F			; STEPS_OLD=STEPS
	ANDWF	STEPS_CHANGE,F		; Keep only rising edge of steps

	BTFSC	STATUS,Z
	GOTO	Step_Out			; No step(s) to do
	MOVF	PORTA,W
	MOVWF	DIRS

; M1
	BTFSS	STEPS_CHANGE,7		; ?M1 step changed
	GOTO	Step_No_M1
	MOVF	INC_STEP,W
	BTFSC	DIRS,0				; M1_DIR=0 -> PTR++
	MOVF	DEC_STEP,W
	ADDWF	M1_STEP,W
	ANDLW	0x07
	MOVWF	M1_STEP
	ADDLW	TABLE_STEP0
	MOVWF	FSR
	MOVF	INDF,W				; Read motor step
	MOVWF	PORTDH				; Store it
; M2
Step_M2:
	BTFSS	STEPS_CHANGE,6
	GOTO	Step_No_M2
	MOVF	INC_STEP,W
	BTFSC	DIRS,1				; M2_DIR=0 -> PTR++
	MOVF	DEC_STEP,W
	ADDWF	M2_STEP,W
	ANDLW	0x07
	MOVWF	M2_STEP
	ADDLW	TABLE_STEP0
	MOVWF	FSR
	MOVF	INDF,W
	MOVWF	PORTCH
; M3
Step_M3:
	BTFSS	STEPS_CHANGE,5
	GOTO	Step_No_M3
	MOVF	INC_STEP,W
	BTFSC	DIRS,2				; M3_DIR=0 -> PTR++
	MOVF	DEC_STEP,W
	ADDWF	M3_STEP,W
	ANDLW	0x07
	MOVWF	M3_STEP
	ADDLW	TABLE_STEP0
	MOVWF	FSR
	MOVF	INDF,W
	MOVWF	PORTDL
; M4
Step_M4:
	BTFSS	STEPS_CHANGE,4
	GOTO	Step_No_M4
	MOVF	INC_STEP,W
	BTFSC	DIRS,3				; M4_DIR=0 -> PTR++
	MOVF	DEC_STEP,W
	ADDWF	M4_STEP,W
	ANDLW	0x07
	MOVWF	M4_STEP
	ADDLW	TABLE_STEP0
	MOVWF	FSR
	MOVF	INDF,W
	MOVWF	PORTCL
Step_Out:
	SWAPF	PORTCH,W
	IORWF	PORTCL,W
	MOVWF	PORTC				; Output motor steps on PORTC
	SWAPF	PORTDH,W
	IORWF	PORTDL,W
	MOVWF	PORTD				; Output motor steps on PORTD
	BTFSC	INTCON, RBIF		;?PORTB INT
	GOTO	Isr_RB				; -> Yes

Isr_CCP1
	BTFSS	PIR1,CCP1IF			;?TIMER
	GOTO	Isr_End				; -> No
	BCF		PIR1,CCP1IF
	MOVLW	0x01
	XORWF	PORTE,F				; TOGGLE S_TIMER pin
	INCF	S_TIMER,F

Isr_End
	SWAPF	STATUS_TEMP,W		;Swap STATUS_TEMP register into W
	MOVWF	STATUS				;Move W into STATUS register
	SWAPF	W_TEMP,F			;Swap W_TEMP
	SWAPF	W_TEMP,W			;Swap W_TEMP into W
	RETFIE

Isr_No_Steps
	CLRF	PORTC
	CLRF	PORTD
	CLRF	STEPS_OLD
	GOTO	Isr_CCP1

Step_No_M1
	BTFSS	PIR1,CCP1IF			;?TIMER
	GOTO	Step_No_M1W			; -> No
	BCF		PIR1,CCP1IF
	MOVLW	0x01
	XORWF	PORTE,F				; TOGGLE S_TIMER pin
	INCF	S_TIMER,F
	NOP		
	GOTO	Step_M2
Step_No_M1W
	NOP							; Just wait...
	NOP
	NOP
	NOP
	GOTO	Step_M2

Step_No_M2
	BTFSS	PIR1,CCP1IF			;?TIMER
	GOTO	Step_No_M2W			; -> No
	BCF		PIR1,CCP1IF
	MOVLW	0x01
	XORWF	PORTE,F				; TOGGLE S_TIMER pin
	INCF	S_TIMER,F
	NOP		
	GOTO	Step_M3
Step_No_M2W
	NOP							; Just wait...
	NOP
	NOP
	NOP
	GOTO	Step_M3

Step_No_M3
	BTFSS	PIR1,CCP1IF			;?TIMER
	GOTO	Step_No_M3W			; -> No
	BCF		PIR1,CCP1IF
	MOVLW	0x01
	XORWF	PORTE,F				; TOGGLE S_TIMER pin
	INCF	S_TIMER,F
	NOP		
	GOTO	Step_M4
Step_No_M3W
	NOP							; Just wait...
	NOP
	NOP
	NOP
	GOTO	Step_M4

Step_No_M4
	BTFSS	PIR1,CCP1IF			;?TIMER
	GOTO	Step_No_M4W			; -> No
	BCF		PIR1,CCP1IF
	MOVLW	0x01
	XORWF	PORTE,F				; TOGGLE S_TIMER pin
	INCF	S_TIMER,F
	NOP		
	GOTO	Step_Out
Step_No_M4W
	NOP							; Just wait...
	NOP
	NOP
	NOP
	GOTO	Step_Out

	RETLW	'*'
	RETLW	'*'
	RETLW	' '
	RETLW	'M'
	RETLW	'M'
	RETLW	'2'
	RETLW	'0'
	RETLW	'0'
	RETLW	'1'
	RETLW	'-'
	RETLW	'H'
	RETLW	'L'
	RETLW	' '
	RETLW	'V'
	RETLW	'1'
	RETLW	'.'
	RETLW	'0'
	RETLW	'3'
	RETLW	' '
	RETLW	'5'
	RETLW	'X'
	RETLW	'-'
	RETLW	'P'
	RETLW	'R'
	RETLW	'O'
	RETLW	'J'
	RETLW	'E'
	RETLW	'C'
	RETLW	'T'
	RETLW	' '
	RETLW	'*'
	RETLW	'*'

Init
	BSF		STATUS,RP0		; Bank 1 (80h-FFh)
	MOVLW	0x3F
	MOVWF	TRISA			; Port A in 0..5, out 6,7
	MOVLW	0xFF
	MOVWF	TRISB			; Port B in 0..7
	CLRF	TRISC			; Port C out 0..7
	CLRF	TRISD			; Port D out 0..7
	CLRF	TRISE			; Port E out 0..7

	MOVLW	b'00000000'
	MOVWF	OPTION_REG		; PORTB Pull-up enabled, Timer 0 in timer mode with prescaler 1/2
	MOVLW	b'01001000'
	MOVWF	INTCON			; All int disabled, int portb on change enabled
	MOVLW	b'00000100'
	MOVWF	PIE1			; CCP1 int
	CLRF	PIE2			; No Peripheral 2 int

	MOVLW	0x07
	MOVWF	ADCON1			; All ports digital
	MOVWF	CMCON			; All ports digital
	CLRF	CVRCON			; Disable Ref module
	
	CLRF	STATUS			; Bank 0
	CLRF	PORTC			; no steps
	CLRF	PORTD			; no steps
	CLRF	PORTE			; TIMER=0, S_CHAUFFE=0, BUZZER=0

	CLRF	T1CON			; TIMER1 disabled, internal clock, no prescale
	CLRF	TMR1L
	CLRF	TMR1H

	CLRF	T2CON			; TIMER2 disabled, internal clock, no prescale, no post scale
	CLRF	TMR2

	MOVLW	B'00001011'
	MOVWF	CCP1CON			; CCP1 enabled in compare mode and reset TMR1 and set CCP1IF on match
	; CCPR1H:CCPR1L
	; 0x00C8	-> 20kHz
	; 0x01F4	->  8kHz
	; 0x03E8	->  4kHz

	CLRF	CCP2CON			; CCP2 disabled

	CLRF	ADCON0			; Turn off ADC

Init_Regs
	; Load step tables
	MOVLW	0x05	; b'00000101'
	MOVWF	TABLE_STEP0
	MOVLW	0x04	; b'00000100'
	MOVWF	TABLE_STEP1
	MOVLW	0x06	; b'00000110'
	MOVWF	TABLE_STEP2
	MOVLW	0x02	; b'00000010'
	MOVWF	TABLE_STEP3
	MOVLW	0x0A	; b'00001010'
	MOVWF	TABLE_STEP4
	MOVLW	0x08	; b'00001000'
	MOVWF	TABLE_STEP5
	MOVLW	0x09	; b'00001001'
	MOVWF	TABLE_STEP6
	MOVLW	0x01	; b'00000001'
	MOVWF	TABLE_STEP7

	; Init Steps
	MOVF	TABLE_STEP0,W
	MOVWF	PORTCH
	MOVWF	PORTCL
	MOVWF	PORTDH
	MOVWF	PORTDL

	CLRF	MOTORS
	CLRF	M1_STEP
	CLRF	M2_STEP
	CLRF	M3_STEP
	CLRF	M4_STEP

	CLRF	STEPS_OLD
	CLRF	STEPS_CHANGE

	; Init PWMCLRF	PWM_POS
	CLRF	PWM_VAL_CHANGED
	CLRF	PWM_MAX_CHANGED
	MOVLW	0x01
	MOVWF	S_TIMER

	; Load values from EEPROM

Load_Freq:
	MOVLW	0x03
	CALL	READ_EE				; Read EEPROM @0x03 -> FREQ
	MOVWF	FREQ
	; Test if FREQ=0x00 or 0x01 or 0x02 or 0x03
	ANDLW	0xFC
	BTFSC	STATUS,Z
	GOTO	Load_Full_Step

	; Write EEPROM @0x03 -> FREQ=0x00
	CLRF	FREQ
	BSF		STATUS,RP1			; Bank 2 (100h-17Fh)
	CLRF	EEDATA
	MOVLW	0x03
	CALL	WRITE_EE			; Write EEPROM @0x03 -> FREQ=0x00

Load_Full_Step:
	MOVLW	0x02
	CALL	READ_EE				; Read EEPROM @0x02 -> FULL_STEP
	MOVWF	FULL_STEP
	; Test if FULL_STEP=0x00 or 0x01
	MOVF	FULL_STEP,F
	BTFSC	STATUS,Z
	GOTO	Load_PWM_MAX
	DECF	FULL_STEP,W
	BTFSC	STATUS,Z
	GOTO	Load_PWM_MAX
	; Write EEPROM @0x02 -> FULL_STEP=0x00
	CLRF	FULL_STEP
	BSF		STATUS,RP1			; Bank 2 (100h-17Fh)
	CLRF	EEDATA
	MOVLW	0x02
	CALL	WRITE_EE			; Write EEPROM @0x02 -> FULL_STEP=0x00

Load_PWM_MAX:
	MOVLW	0x01
	CALL	READ_EE				; Read EEPROM @0x01 -> PWM_MAX
	MOVWF	PWM_MAX
	; Test PWM_MAX<100 & PWM_MAX>2
	MOVLW	.99
	SUBWF	PWM_MAX,W
	BTFSC	STATUS,C
	GOTO	Write_PWM_MAX
	MOVLW	.2
	SUBWF	PWM_MAX,W
	BTFSC	STATUS,C
	GOTO	Load_PWM_Val
Write_PWM_MAX:
	MOVLW	0x63
	MOVWF	PWM_MAX				; PWM_MAX=99
	BSF		STATUS,RP1			; Bank 2 (100h-17Fh)
	MOVWF	EEDATA
	MOVLW	0x01
	CALL	WRITE_EE			; Write EEPROM @0x01 -> PWM_MAX=99

Load_PWM_Val:
	MOVLW	0x00
	CALL	READ_EE				; Read EEPROM @0x00 -> PWM_VAL
	MOVWF	PWM_VAL

	; Test PWM_VAL<PWM_MAX
	SUBWF	PWM_MAX,W
	BTFSC	STATUS,C
	GOTO	Change_Config
	; Write EEPROM @0x00 -> PWM_VAL=1
	MOVLW	0x01
	MOVWF	PWM_VAL
	BSF		STATUS,RP1			; Bank 2 (100h-17Fh)
	MOVWF	EEDATA
	MOVLW	0x00
	CALL	WRITE_EE			; Write EEPROM @0x00 -> PWM_VAL=1

;**** CONFIG CHANGE ****
Change_Config:
	BTFSS	PORTB,2				; PC_MAN
	GOTO	Bip_Config			; change config only if PC_MAN=0
	; PC_MAN in MAN MODE
	CLRF	STATUS				; Bank 0 (00h-7Fh)
Change_Freq:
	BTFSS	PORTA,4				; BP-
	BTFSS	PORTA,5				; BP+
	GOTO	Change_Full_Step
	; BP- pressed, BP+ not pressed
	INCF	FREQ,W
	ANDLW	0x03
	MOVWF	FREQ
	; Write EEPROM @0x03 -> FREQ
	BSF		STATUS,RP1			; Bank 2 (100h-17Fh)
	MOVWF	EEDATA
	MOVLW	0x03
	CALL	WRITE_EE			; Write EEPROM @0x03 -> FREQ=0x00
	GOTO	Change_Wait_BP

; CHANGE FULL/HALF
Change_Full_Step:
	BTFSS	PORTA,4				; BP-
	BTFSC	PORTA,5				; BP+
	GOTO	Reset_PWM_Max
	; BP- pressed, BP+ pressed
	MOVLW	0x01
	XORWF	FULL_STEP,F
	; Write EEPROM @0x02 -> FULL_STEP
	MOVF	FULL_STEP,W
	BSF		STATUS,RP1			; Bank 2 (100h-17Fh)
	MOVWF	EEDATA
	MOVLW	0x02
	CALL	WRITE_EE			; Write EEPROM @0x02 -> FULL_STEP
	GOTO	Change_Wait_BP

; RESET PWM_MAX
Reset_PWM_Max:
	BTFSS	PORTA,5				; BP+
	BTFSS	PORTA,4				; BP-
	GOTO	Bip_Config
	; BP- not pressed, BP+ pressed
	MOVLW	.99
	MOVWF	PWM_MAX				; PWM_MAX=99
	; Write EEPROM @0x01 -> PWM_MAX=99
	BSF		STATUS,RP1			; Bank 2 (100h-17Fh)
	MOVWF	EEDATA
	MOVLW	0x01
	CALL	WRITE_EE			; Write EEPROM @0x01 -> PWM_MAX=99
	BSF		PORTE,2				; Buzz on
	BSF		PWM_MAX_CHANGED,0

; Wait for BP+ and/or BP- to be released
Change_Wait_BP:
	BTFSC	PORTA,4				; BP-
	BTFSS	PORTA,5				; BP+
	GOTO	Change_Wait_BP

	BTFSS	PWM_MAX_CHANGED,0	; PWM_MAX reseted ?
	GOTO	Bip_Config			;  -> no
	CLRF	PWM_MAX_CHANGED
	BCF		PORTE,2				; Buzz off
	MOVLW	0x0D
	MOVWF	WAITH
	MOVLW	0x3C
	MOVWF	WAITL
	CALL	WAIT				; wait...


;**** CONFIG BIP ****
Bip_Config:
	; BIP with the choosed FREQ
	INCF	FREQ,W
	CALL	BIP

	; Wait
	MOVLW	0x0D
	MOVWF	WAITH
	MOVLW	0x3C
	MOVWF	WAITL
	CALL	WAIT				; wait...

	; BIP with FULL/HALF STEP
	INCF	FULL_STEP,W
	CALL	BIP


;**** SET FREQ ****
	MOVF	FREQ,W
	BTFSS	STATUS,Z
	GOTO	Set_FREQ_1
	; FREQ=0  => 0x03E8	->  4kHz
	MOVLW	0x03
	MOVWF	CCPR1H
	MOVLW	0xE8
	MOVWF	CCPR1L
	; Keyfilter = 0,3s
	MOVLW	.3
	MOVWF	KEY_FILTER
	MOVWF	KEY_FILTER_VAL
	GOTO	Set_Full_Step
Set_FREQ_1:
	DECF	FREQ,W
	BTFSS	STATUS,Z
	GOTO	Set_FREQ_2
	; FREQ=1  => 0x01F4	->  8kHz
	MOVLW	0x01
	MOVWF	CCPR1H
	MOVLW	0xF4
	MOVWF	CCPR1L
	; Keyfilter = 0,3s
	MOVLW	.6
	MOVWF	KEY_FILTER
	MOVWF	KEY_FILTER_VAL
	GOTO	Set_Full_Step
Set_FREQ_2:
	SUBLW	0x01
	BTFSS	STATUS,Z
	GOTO	Set_FREQ_3
	; FREQ=2  => 0x00C8	->  20kHz
	CLRF	CCPR1H
	MOVLW	0xC8
	MOVWF	CCPR1L
	; Keyfilter = 0,3s
	MOVLW	.15
	MOVWF	KEY_FILTER
	MOVWF	KEY_FILTER_VAL
	GOTO	Set_Full_Step
Set_FREQ_3:
	CLRF	CCP1CON				; CCP1 disabled
	MOVLW	0xFF
	MOVWF	KEY_FILTER
	MOVWF	KEY_FILTER_VAL

;**** SET VALUE FULL/HALF STEP ****
Set_Full_Step:
	MOVLW	0x01				; half step
	MOVWF	INC_STEP
	MOVLW	0xFF				; half step
	MOVWF	DEC_STEP
	BTFSS	FULL_STEP,0
	INCF	INC_STEP,F			; full step
	BTFSS	FULL_STEP,0
	DECF	DEC_STEP,F			; full step

;********************
;****   START   *****
;********************
Start:
	BSF 	INTCON,GIE			; enable interrupt
	BSF		T1CON,TMR1ON		; Timer 1 enabled

Main_Loop:
	MOVF	PORTB,W
	ANDLW	0x01
	XORWF	MOTORS,W
	BTFSC	STATUS,Z
	GOTO	Main_Loop1
	XORWF	MOTORS,F
	BSF		INTCON, RBIF
Main_Loop1:
	MOVLW	0xFE
	ANDWF	S_TIMER,W
	BTFSC	STATUS,Z			; Loop while S_TIMER=0 or 1
	GOTO	Main_Loop
	CLRF	S_TIMER

;****   DO PWM   *****
Do_PWM:
	BTFSS	PORTB,2				; ETAT_MANPC
	GOTO	PWM_ETAT_PC			; ETAT_MANPC=0 -> goto PWM_ETAT_PC

;PWM IN MANUAL MODE
	INCF	PWM_POS,F			; PWM_POS=PWM_POS+1
	MOVF	PWM_POS,W
	SUBWF	PWM_VAL,W
	BTFSS	STATUS,C
	BCF		PORTE,1				; PWM_POS>PWM_VAL => S_CHAUFFE=0
	BTFSC	STATUS,C
	BSF		PORTE,1				; PWM_POS<PWM_VAL => S_CHAUFFE=1

	MOVF	PWM_POS,W
	XORLW	0x64
	BTFSS	STATUS,Z
	GOTO	End_PWM

	CLRF	PWM_POS				; ?PWM_POS=100 -> PWM_POS=0
	DECFSZ	KEY_FILTER,F
	GOTO	End_PWM
	BCF		PORTE,2				; BUZZ off
	INCF	KEY_FILTER,F
	BTFSC	PORTA,5				; ?BP+
	GOTO	PWM_Man_BP_M
	BTFSC	PORTA,4				; ?BP-
	GOTO	PWM_Man_BP_P
	MOVF	PWM_VAL,W			; if BP+ pressed and BP- pressed
	MOVWF	PWM_MAX				;  -> PWM_MAX=PWM_VAL
	BSF		PWM_MAX_CHANGED,0	; PWM_MAX_CHANGED=1
PWM_Man_Exit_Bip:
	BSF		PORTE,2				; Buzz on
	MOVF	KEY_FILTER_VAL,W
	MOVWF	KEY_FILTER
	GOTO	End_PWM

PWM_Man_BP_P:					; BP+ is pressed
	INCF	PWM_VAL,W
	SUBWF	PWM_MAX,W
	BTFSS	STATUS,C
	GOTO	PWM_Man_Exit_Bip
	INCF	PWM_VAL,F
	BSF		PWM_VAL_CHANGED,0	; PWM_VAL has changed
	MOVF	KEY_FILTER_VAL,W
	MOVWF	KEY_FILTER
	GOTO	End_PWM

PWM_Man_BP_M:
	BTFSC	PORTA,4				; BP- is pressed?
	GOTO	End_PWM				;  -> no exit
	BSF		PWM_VAL_CHANGED,0	; PWM_VAL has changed
	MOVF	KEY_FILTER_VAL,W
	MOVWF	KEY_FILTER
	DECFSZ	PWM_VAL,F			; yes PWM_VAL=PWM_VAL-1
	GOTO	End_PWM
	INCF	PWM_VAL,F			;   -> PWM_VAL=1
	BSF		PORTE,2				; Buzz on
	GOTO	End_PWM

;PWM IN PC MODE
PWM_ETAT_PC:
	BTFSS	PORTB,1				; if CHAUFFE_PC=0
	GOTO	PWM_PC_Off			;   -> PWM_PC_Off
	INCF	PWM_PC,W			; PWM_PC+1
	SUBWF	PWM_MAX,W			; if PWM_PC+1>PWM_MAX
	BTFSS	STATUS,C
	GOTO	PWM_PC_Over			;   -> PWM_PC_Over
	INCF	PWM_PC,F			; PWM_PC=PWM_PC+1
	BSF		PORTE,1				; S_CHAUFFE=1
	GOTO	End_PWM
PWM_PC_Off:
	CLRF	PWM_PC				; PWM_PC=0
	BCF		PORTE,2				; BUZZ Off
PWM_PC_Over:
	BCF		PORTE,1				; S_CHAUFFE=0
;PWM_PC_Over:
;	BCF		PORTE,1				; S_CHAUFFE=0
;	BSF		PORTE,2				; BUZZ On
;	GOTO	End_PWM
;PWM_PC_Off:
;	CLRF	PWM_PC				; PWM_PC=0
;	BCF		PORTE,1				; S_CHAUFFE=0
;	BCF		PORTE,2				; BUZZ Off

;****   MOTORS OFF  *****
End_PWM:
	BTFSC	PORTB,0				; ?MOTEUR_ON
	GOTO	Main_Loop

	BTFSS	PWM_VAL_CHANGED,0	; if PWM_VAL_CHANGED=0
	GOTO	End_PWM_1			;  -> goto Stop_Steps_1
	CLRF	PWM_VAL_CHANGED		; PWM_VAL_CHANGED=0
	; Write EEPROM @0x00 -> PWM_VAL
	MOVF	PWM_VAL,W
	BSF		STATUS,RP1			; Bank 2 (100h-17Fh)
	MOVWF	EEDATA
	MOVLW	0x00
	CALL	WRITE_EE			; Write EEPROM @0x00 -> PWM_VAL
End_PWM_1:
	BTFSS	PWM_MAX_CHANGED,0
	GOTO	Main_Loop
	CLRF	PWM_MAX_CHANGED
	; Write EEPROM @0x01 -> PWM_MAX
	MOVF	PWM_MAX,W
	BSF		STATUS,RP1			; Bank 2 (100h-17Fh)
	MOVWF	EEDATA
	MOVLW	0x01
	CALL	WRITE_EE			; Write EEPROM @0x01 -> PWM_MAX
	GOTO	Main_Loop

;*********************
; BIP NBR_BIP times
;*********************
BIP:
	CLRF	STATUS
	MOVWF	NBR_BIP
L92:
	MOVLW	0x04
	MOVWF	WAITH
	MOVLW	0x69
	MOVWF	WAITL
	CALL	WAIT
	BSF		PORTE,2			; Buzz on
	MOVLW	0x04
	MOVWF	WAITH
	MOVLW	0x69
	MOVWF	WAITL
	CALL	WAIT
	BCF		PORTE,2			; Buzz off
	DECFSZ	NBR_BIP,F
	GOTO	L92
	RETURN

;**************************
; WAIT WAITH:WAITL * 195s
;**************************
WAIT:
	CLRF	STATUS
	MOVF	WAITL,F
	BTFSC	STATUS,Z
	DECF	WAITH,F
	DECF	WAITL,F
	CLRF	WAITS
	DECFSZ	WAITS,F
	GOTO	$-1
	MOVF	WAITH,W
	IORWF	WAITL,W
	BTFSC	STATUS,Z
	RETURN
	GOTO	WAIT

;*********************
; Read EEPROM
; IN  W=ADR
; OUT W=EEDATA, Bank 0
;*********************
READ_EE
	BSF		STATUS,RP1		; Bank 2 (100h-17Fh)
	MOVWF	EEADR
	BSF		STATUS,RP0		; Bank 3 (180h-1FFh)
	BCF		EECON1,EEPGD
	BSF		EECON1,RD
	BCF		STATUS,RP0		; Bank 2 (100h-17Fh)
	MOVF 	EEDATA,W
	BCF 	STATUS,RP1		; Bank 0 (00h-7Fh)
	RETURN

;*********************
; Write EEPROM
; IN  W=ADR
;     EEDATA
;     Bank 2
; OUT Bank 0
;*********************
WRITE_EE
	BSF		STATUS,RP0		; Bank 3 (180h-1FFh)
	BTFSC	EECON1,WR		; Wait for previous write to end
	GOTO	$-1
	BCF		STATUS,RP0		; Bank 2 (100h-17Fh)
	MOVWF	EEADR
	BSF		STATUS,RP0		; Bank 3 (180h-1FFh)
	BCF		EECON1,EEPGD
	BCF		STATUS,C
	BTFSC	INTCON,GIE
	BSF		STATUS,C
	BCF		INTCON,GIE
	BSF		EECON1,WREN
	MOVLW	0x55
	MOVWF	EECON2
	MOVLW	0xAA
	MOVWF	EECON2
	BSF		EECON1,WR
	BCF		EECON1,WREN
	BTFSC	STATUS,C
	BSF		INTCON,GIE
	BTFSC	EECON1,WR		; Wait for previous write to end
	GOTO	$-1
	BCF		STATUS,RP0
	BCF		STATUS,RP1		; Bank 0 (00h-7Fh)
	RETURN


;*********************
; EEPROM
; Default
;*********************
;	org		0x2100
;	de	0xFF		; PWM value , default=0x01
;	de	0xFF		; PWM max   , default=0x63
;	de	0xFF		; STEP      , default=0x00
;	de	0xFF		; FREQ value, default=0x00
;	de	0x00
;	de	'*'
;	de	'*'
;	de	' '
;	de	'M'
;	de	'M'
;	de	'2'
;	de	'0'
;	de	'0'
;	de	'1'
;	de	'-'
;	de	'H'
;	de	'L'
;	de	' '
;	de	'V'
;	de	'1'
;	de	'.'
;	de	'0'
;	de	'3'
;	de	' '
;	de	'5'
;	de	'X'
;	de	'-'
;	de	'P'
;	de	'R'
;	de	'O'
;	de	'J'
;	de	'E'
;	de	'C'
;	de	'T'
;	de	' '
;	de	'*'
;	de	'*'
;	de	0x00
	END